@@ -4,10 +4,11 @@
 # It also implements keyboard traversal of menus and implements a few
 # other utility procedures related to menus.
 #
-# @(#) menu.tcl 1.52 95/07/28 09:35:55
+# RCS: @(#) $Id: menu.tcl,v 1.5 1999/04/16 01:51:26 stanton Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
-# Copyright (c) 1994-1995 Sun Microsystems, Inc.
+# Copyright (c) 1994-1997 Sun Microsystems, Inc.
+# Copyright (c) 1998-1999 by Scriptics Corporation.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -27,6 +28,10 @@
 # inMenubutton -	The name of the menubutton widget containing
 #			the mouse, or an empty string if the mouse is
 #			not over any menubutton.
+# menuBar -		The name of the menubar that is the root
+#			of the cascade hierarchy which is currently
+#			posted. This is null when there is no menu currently
+#			being pulled down from a menu bar.
 # oldGrab -		Window that had the grab before a menu was posted.
 #			Used to restore the grab state after the menu
 #			is unposted.  Empty string means there was no
@@ -42,23 +47,30 @@
 # window -		When the mouse is over a menu, this holds the
 #			name of the menu;  it's cleared when the mouse
 #			leaves the menu.
+# tearoff -		Whether the last menu posted was a tearoff or not.
+#			This is true always for unix, for tearoffs for Mac
+#			and Windows.
+# activeMenu -		This is the last active menu for use
+#			with the <<MenuSelect>> virtual event.
+# activeItem -		This is the last active menu item for
+#			use with the <<MenuSelect>> virtual event.
 #-------------------------------------------------------------------------
 
 #-------------------------------------------------------------------------
 # Overall note:
-# This file is tricky because there are four different ways that menus
+# This file is tricky because there are five different ways that menus
 # can be used:
 #
-# 1. As a pulldown from a menubutton.  This is the most common usage.
-#    In this style, the variable tkPriv(postedMb) identifies the posted
-#    menubutton.
+# 1. As a pulldown from a menubutton. In this style, the variable 
+#    tkPriv(postedMb) identifies the posted menubutton.
 # 2. As a torn-off menu copied from some other menu.  In this style
-#    tkPriv(postedMb) is empty, and the top-level menu is no
-#    override-redirect.
-# 3. As an option menu, triggered from an option menubutton.  In thi
+#    tkPriv(postedMb) is empty, and menu's type is "tearoff".
+# 3. As an option menu, triggered from an option menubutton.  In this
 #    style tkPriv(postedMb) identifies the posted menubutton.
 # 4. As a popup menu.  In this style tkPriv(postedMb) is empty and
-#    the top-level menu is override-redirect.
+#    the top-level menu's type is "normal".
+# 5. As a pulldown from a menubar. The variable tkPriv(menubar) has
+#    the owning menubar, and the menu itself is of type "normal".
 #
 # The various binding procedures use the  state described above to
 # distinguish the various cases and take different actions in each
@@ -78,7 +90,7 @@ bind Menubutton <Leave> {
     tkMbLeave %W
 }
 bind Menubutton <1> {
-    if {$tkPriv(inMenubutton) != ""} {
+    if {[string compare $tkPriv(inMenubutton) ""]} {
 	tkMbPost $tkPriv(inMenubutton) %X %Y
     }
 }
@@ -95,51 +107,60 @@ bind Menubutton <space> {
     tkMbPost %W
     tkMenuFirstEntry [%W cget -menu]
 }
-bind Menubutton <Return> {
-    tkMbPost %W
-    tkMenuFirstEntry [%W cget -menu]
-}
 
 # Must set focus when mouse enters a menu, in order to allow
 # mixed-mode processing using both the mouse and the keyboard.
+# Don't set the focus if the event comes from a grab release,
+# though:  such an event can happen after as part of unposting
+# a cascaded chain of menus, after the focus has already been
+# restored to wherever it was before menu selection started.
 
 bind Menu <FocusIn> {}
+
 bind Menu <Enter> {
     set tkPriv(window) %W
-    focus %W
+    if {![string compare [%W cget -type] "tearoff"]} {
+      if {[string compare "%m" "NotifyUngrab"]} {
+          if {![string compare $tcl_platform(platform) "unix"]} {
+		tk_menuSetFocus %W
+	    }
+	}
+    }
+    tkMenuMotion %W %x %y %s
 }
+
 bind Menu <Leave> {
     tkMenuLeave %W %X %Y %s
 }
 bind Menu <Motion> {
-    tkMenuMotion %W %y %s
+    tkMenuMotion %W %x %y %s
 }
 bind Menu <ButtonPress> {
     tkMenuButtonDown %W
 }
 bind Menu <ButtonRelease> {
-    tkMenuInvoke %W
+   tkMenuInvoke %W 1
 }
 bind Menu <space> {
-    tkMenuInvoke %W
+    tkMenuInvoke %W 0
 }
 bind Menu <Return> {
-    tkMenuInvoke %W
+    tkMenuInvoke %W 0
 }
 bind Menu <Escape> {
     tkMenuEscape %W
 }
 bind Menu <Left> {
-    tkMenuLeftRight %W left
+    tkMenuLeftArrow %W
 }
 bind Menu <Right> {
-    tkMenuLeftRight %W right
+    tkMenuRightArrow %W
 }
 bind Menu <Up> {
-    tkMenuNextEntry %W -1
+    tkMenuUpArrow %W
 }
 bind Menu <Down> {
-    tkMenuNextEntry %W +1
+    tkMenuDownArrow %W
 }
 bind Menu <KeyPress> {
     tkTraverseWithinMenu %W %A
@@ -148,11 +169,22 @@ bind Menu <KeyPress> {
 # The following bindings apply to all windows, and are used to
 # implement keyboard menu traversal.
 
-bind all <Alt-KeyPress> {
-    tkTraverseToMenu %W %A
-}
-bind all <F10> {
-    tkFirstMenu %W
+if {![string compare $tcl_platform(platform) "unix"]} {
+    bind all <Alt-KeyPress> {
+	tkTraverseToMenu %W %A
+    }
+
+    bind all <F10> {
+	tkFirstMenu %W
+    }
+} else {
+    bind Menubutton <Alt-KeyPress> {
+	tkTraverseToMenu %W %A
+    }
+
+    bind Menubutton <F10> {
+	tkFirstMenu %W
+    }
 }
 
 # tkMbEnter --
@@ -167,11 +199,11 @@ bind all <F10> {
 proc tkMbEnter w {
     global tkPriv
 
-    if {$tkPriv(inMenubutton) != ""} {
+    if {[string compare $tkPriv(inMenubutton) ""]} {
 	tkMbLeave $tkPriv(inMenubutton)
     }
     set tkPriv(inMenubutton) $w
-    if {[$w cget -state] != "disabled"} {
+    if {[string compare [$w cget -state] "disabled"]} {
 	$w configure -state active
     }
 }
@@ -187,10 +219,10 @@ proc tkMbLeave w {
     global tkPriv
 
     set tkPriv(inMenubutton) {}
-    if ![winfo exists $w] {
+    if {![winfo exists $w]} {
 	return
     }
-    if {[$w cget -state] == "active"} {
+    if {![string compare [$w cget -state] "active"]} {
 	$w configure -state normal
     }
 }
@@ -208,45 +240,119 @@ proc tkMbLeave w {
 #			of the menubutton is used for an option menu.
 
 proc tkMbPost {w {x {}} {y {}}} {
-    global tkPriv
-    if {([$w cget -state] == "disabled") || ($w == $tkPriv(postedMb))} {
+    global tkPriv errorInfo
+    global tcl_platform
+
+    if {![string compare [$w cget -state] "disabled"] ||
+      ![string compare $w $tkPriv(postedMb)]} {
 	return
     }
     set menu [$w cget -menu]
-    if {($menu == "") || ([$menu index last] == "none")} {
+    if {![string compare $menu ""]} {
 	return
     }
-    if ![string match $w.* $menu] {
+    set tearoff [expr {![string compare $tcl_platform(platform) "unix"] \
+          || ![string compare [$menu cget -type] "tearoff"]}]
+    if {[string first $w $menu] != 0} {
 	error "can't post $menu:  it isn't a descendant of $w (this is a new requirement in Tk versions 3.0 and later)"
     }
     set cur $tkPriv(postedMb)
-    if {$cur != ""} {
+    if {[string compare $cur ""]} {
 	tkMenuUnpost {}
     }
     set tkPriv(cursor) [$w cget -cursor]
     set tkPriv(relief) [$w cget -relief]
     $w configure -cursor arrow
     $w configure -relief raised
+
     set tkPriv(postedMb) $w
     set tkPriv(focus) [focus]
     $menu activate none
+    tkGenerateMenuSelect $menu
 
     # If this looks like an option menubutton then post the menu so
     # that the current entry is on top of the mouse.  Otherwise post
     # the menu just below the menubutton, as for a pull-down.
 
-    if {([$w cget -indicatoron] == 1) && ([$w cget -textvariable] != "")} {
-	if {$y == ""} {
-	    set x [expr [winfo rootx $w] + [winfo width $w]/2]
-	    set y [expr [winfo rooty $w] + [winfo height $w]/2]
-	}
-	tkPostOverPoint $menu $x $y [tkMenuFindName $menu [$w cget -text]]
-    } else {
-	$menu post [winfo rootx $w] [expr [winfo rooty $w]+[winfo height $w]]
+    update idletasks
+    if {[catch {
+    	 switch [$w cget -direction] {
+    	    above {
+    	    	set x [winfo rootx $w]
+    	    	set y [expr {[winfo rooty $w] - [winfo reqheight $menu]}]
+    	    	$menu post $x $y
+    	    }
+    	    below {
+    	    	set x [winfo rootx $w]
+    	    	set y [expr {[winfo rooty $w] + [winfo height $w]}]
+    	    	$menu post $x $y
+    	    }
+    	    left {
+    	    	set x [expr {[winfo rootx $w] - [winfo reqwidth $menu]}]
+    	    	set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
+    	    	set entry [tkMenuFindName $menu [$w cget -text]]
+    	    	if {[$w cget -indicatoron]} {
+		    if {$entry == [$menu index last]} {
+		    	incr y [expr {-([$menu yposition $entry] \
+			    	+ [winfo reqheight $menu])/2}]
+		    } else {
+		    	incr y [expr {-([$menu yposition $entry] \
+			        + [$menu yposition [expr {$entry+1}]])/2}]
+		    }
+    	    	}
+    	    	$menu post $x $y
+              if {[string compare $entry {}] && [string compare [$menu entrycget $entry -state] "disabled"]} {
+    	    	    $menu activate $entry
+		    tkGenerateMenuSelect $menu
+    	    	}
+    	    }
+    	    right {
+    	    	set x [expr {[winfo rootx $w] + [winfo width $w]}]
+    	    	set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
+    	    	set entry [tkMenuFindName $menu [$w cget -text]]
+    	    	if {[$w cget -indicatoron]} {
+		    if {$entry == [$menu index last]} {
+		    	incr y [expr {-([$menu yposition $entry] \
+			    	+ [winfo reqheight $menu])/2}]
+		    } else {
+		    	incr y [expr {-([$menu yposition $entry] \
+			        + [$menu yposition [expr {$entry+1}]])/2}]
+		    }
+    	    	}
+    	    	$menu post $x $y
+              if {[string compare $entry {}] && [string compare [$menu entrycget $entry -state] "disabled"]} {
+    	    	    $menu activate $entry
+		    tkGenerateMenuSelect $menu
+    	    	}
+    	    }
+    	    default {
+    	    	if {[$w cget -indicatoron]} {
+                  if {![string compare $y {}]} {
+			set x [expr {[winfo rootx $w] + [winfo width $w]/2}]
+			set y [expr {[winfo rooty $w] + [winfo height $w]/2}]
+	    	    }
+	            tkPostOverPoint $menu $x $y [tkMenuFindName $menu [$w cget -text]]
+		} else {
+	    	    $menu post [winfo rootx $w] [expr {[winfo rooty $w]+[winfo height $w]}]
+    	    	}  
+    	    }
+    	 }
+     } msg]} {
+	# Error posting menu (e.g. bogus -postcommand). Unpost it and
+	# reflect the error.
+	
+	set savedInfo $errorInfo
+	tkMenuUnpost {}
+	error $msg $savedInfo
+
+    }
+
+    set tkPriv(tearoff) $tearoff
+    if {$tearoff != 0} {
+    	focus $menu
+    	tkSaveGrabInfo $w
+    	grab -global $w
     }
-    focus $menu
-    tkSaveGrabInfo $w
-    grab -global $w
 }
 
 # tkMenuUnpost --
@@ -266,6 +372,7 @@ proc tkMbPost {w {x {}} {y {}}} {
 #			is a posted menubutton.
 
 proc tkMenuUnpost menu {
+    global tcl_platform
     global tkPriv
     set mb $tkPriv(postedMb)
 
@@ -280,16 +387,17 @@ proc tkMenuUnpost menu {
     # what was posted.
 
     catch {
-	if {$mb != ""} {
+      if {[string compare $mb ""]} {
 	    set menu [$mb cget -menu]
 	    $menu unpost
 	    set tkPriv(postedMb) {}
 	    $mb configure -cursor $tkPriv(cursor)
 	    $mb configure -relief $tkPriv(relief)
-	} elseif {$tkPriv(popup) != ""} {
+      } elseif {[string compare $tkPriv(popup) ""]} {
 	    $tkPriv(popup) unpost
 	    set tkPriv(popup) {}
-	} elseif {[wm overrideredirect $menu]} {
+      } elseif {[string compare [$menu cget -type] "menubar"]
+              && [string compare [$menu cget -type] "tearoff"]} {
 	    # We're in a cascaded sub-menu from a torn-off menu or popup.
 	    # Unpost all the menus up to the toplevel one (but not
 	    # including the top-level torn-off one) and deactivate the
@@ -297,37 +405,43 @@ proc tkMenuUnpost menu {
 
 	    while 1 {
 		set parent [winfo parent $menu]
-		if {([winfo class $parent] != "Menu")
+              if {[string compare [winfo class $parent] "Menu"]
 			|| ![winfo ismapped $parent]} {
 		    break
 		}
 		$parent activate none
 		$parent postcascade none
-		if {![wm overrideredirect $parent]} {
+		tkGenerateMenuSelect $parent
+		set type [$parent cget -type]
+              if {![string compare $type "menubar"] ||
+                  ![string compare $type "tearoff"]} {
 		    break
 		}
 		set menu $parent
 	    }
-	    $menu unpost
+          if {[string compare [$menu cget -type] "menubar"]} {
+		$menu unpost
+	    }
 	}
     }
 
-    # Release grab, if any, and restore the previous grab, if there
-    # was one.
-
-    if {$menu != ""} {
-	set grab [grab current $menu]
-	if {$grab != ""} {
-	    grab release $grab
+    if {($tkPriv(tearoff) != 0) || [string compare $tkPriv(menuBar) ""]} {
+    	# Release grab, if any, and restore the previous grab, if there
+    	# was one.
+      if {[string compare $menu ""]} {
+	    set grab [grab current $menu]
+          if {[string compare $grab ""]} {
+		grab release $grab
+	    }
 	}
-    }
-    if {$tkPriv(oldGrab) != ""} {
-	if {$tkPriv(grabStatus) == "global"} {
-	    grab set -global $tkPriv(oldGrab)
-	} else {
-	    grab set $tkPriv(oldGrab)
+	tkRestoreOldGrab
+      if {[string compare $tkPriv(menuBar) ""]} {
+	    $tkPriv(menuBar) configure -cursor $tkPriv(cursor)
+	    set tkPriv(menuBar) {}
+	}
+      if {[string compare $tcl_platform(platform) "unix"]} {
+	    set tkPriv(tearoff) 0
 	}
-	set tkPriv(oldGrab) ""
     }
 }
 
@@ -345,18 +459,21 @@ proc tkMenuUnpost menu {
 proc tkMbMotion {w upDown rootx rooty} {
     global tkPriv
 
-    if {$tkPriv(inMenubutton) == $w} {
+    if {![string compare $tkPriv(inMenubutton) $w]} {
 	return
     }
     set new [winfo containing $rootx $rooty]
-    if {($new != $tkPriv(inMenubutton)) && (($new == "")
-	    || ([winfo toplevel $new] == [winfo toplevel $w]))} {
-	if {$tkPriv(inMenubutton) != ""} {
+    if {[string compare $new $tkPriv(inMenubutton)]
+          && (![string compare $new ""]
+          || ![string compare [winfo toplevel $new] [winfo toplevel $w]])} {
+      if {[string compare $tkPriv(inMenubutton) ""]} {
 	    tkMbLeave $tkPriv(inMenubutton)
 	}
-	if {($new != "") && ([winfo class $new] == "Menubutton")
-		&& ([$new cget -indicatoron] == 0)} {
-	    if {$upDown == "down"} {
+      if {[string compare $new ""]
+              && ![string compare [winfo class $new] "Menubutton"]
+		&& ([$new cget -indicatoron] == 0)
+		&& ([$w cget -indicatoron] == 0)} {
+          if {![string compare $upDown "down"]} {
 		tkMbPost $new $rootx $rooty
 	    } else {
 		tkMbEnter $new
@@ -375,8 +492,13 @@ proc tkMbMotion {w upDown rootx rooty} {
 
 proc tkMbButtonUp w {
     global tkPriv
+    global tcl_platform
 
-    if  {($tkPriv(postedMb) == $w) && ($tkPriv(inMenubutton) == $w)} {
+    set menu [$w cget -menu]
+    set tearoff [expr {($tcl_platform(platform) == "unix") \
+	    || (($menu != {}) && ([$menu cget -type] == "tearoff"))}]
+    if {($tearoff != 0) && ($tkPriv(postedMb) == $w) 
+	    && ($tkPriv(inMenubutton) == $w)} {
 	tkMenuFirstEntry [$tkPriv(postedMb) cget -menu]
     } else {
 	tkMenuUnpost {}
@@ -392,13 +514,23 @@ proc tkMbButtonUp w {
 #
 # Arguments:
 # menu -		The menu window.
+# x -			The x position of the mouse.
 # y -			The y position of the mouse.
 # state -		Modifier state (tells whether buttons are down).
 
-proc tkMenuMotion {menu y state} {
+proc tkMenuMotion {menu x y state} {
     global tkPriv
-    if {$menu == $tkPriv(window)} {
-	$menu activate @$y
+    if {![string compare $menu $tkPriv(window)]} {
+      if {![string compare [$menu cget -type] "menubar"]} {
+	    if {[info exists tkPriv(focus)] && \
+                  [string compare $menu $tkPriv(focus)]} {
+		$menu activate @$x,$y
+		tkGenerateMenuSelect $menu
+	    }
+	} else {
+	    $menu activate @$x,$y
+	    tkGenerateMenuSelect $menu
+	}
     }
     if {($state & 0x1f00) != 0} {
 	$menu postcascade active
@@ -422,17 +554,38 @@ proc tkMenuMotion {menu y state} {
 
 proc tkMenuButtonDown menu {
     global tkPriv
+    global tcl_platform
     $menu postcascade active
-    if {$tkPriv(postedMb) != ""} {
+    if {[string compare $tkPriv(postedMb) ""]} {
 	grab -global $tkPriv(postedMb)
     } else {
-	while {[wm overrideredirect $menu]
-		&& ([winfo class [winfo parent $menu]] == "Menu")
+      while {![string compare [$menu cget -type] "normal"]
+              && ![string compare [winfo class [winfo parent $menu]] "Menu"]
 		&& [winfo ismapped [winfo parent $menu]]} {
 	    set menu [winfo parent $menu]
 	}
-	tkSaveGrabInfo $menu
-	grab -global $menu
+
+      if {![string compare $tkPriv(menuBar) {}]} {
+	    set tkPriv(menuBar) $menu
+	    set tkPriv(cursor) [$menu cget -cursor]
+	    $menu configure -cursor arrow
+        }
+
+	# Don't update grab information if the grab window isn't changing.
+	# Otherwise, we'll get an error when we unpost the menus and
+	# restore the grab, since the old grab window will not be viewable
+	# anymore.
+
+      if {[string compare $menu [grab current $menu]]} {
+	    tkSaveGrabInfo $menu
+	}
+
+	# Must re-grab even if the grab window hasn't changed, in order
+	# to release the implicit grab from the button press.
+
+      if {![string compare $tcl_platform(platform) "unix"]} {
+	    grab -global $menu
+	}
     }
 }
 
@@ -449,15 +602,16 @@ proc tkMenuButtonDown menu {
 proc tkMenuLeave {menu rootx rooty state} {
     global tkPriv
     set tkPriv(window) {}
-    if {[$menu index active] == "none"} {
+    if {![string compare [$menu index active] "none"]} {
 	return
     }
-    if {([$menu type active] == "cascade")
-	    && ([winfo containing $rootx $rooty]
-	    == [$menu entrycget active -menu])} {
+    if {![string compare [$menu type active] "cascade"]
+          && ![string compare [winfo containing $rootx $rooty] \
+                  [$menu entrycget active -menu]]} {
 	return
     }
     $menu activate none
+    tkGenerateMenuSelect $menu
 }
 
 # tkMenuInvoke --
@@ -467,15 +621,35 @@ proc tkMenuLeave {menu rootx rooty state} {
 #
 # Arguments:
 # w -			Name of the menu widget.
+# buttonRelease -	1 means this procedure is called because of
+#			a button release;  0 means because of keystroke.
 
-proc tkMenuInvoke w {
-    if {[$w type active] == "cascade"} {
+proc tkMenuInvoke {w buttonRelease} {
+    global tkPriv
+
+    if {$buttonRelease && ![string compare $tkPriv(window) {}]} {
+	# Mouse was pressed over a menu without a menu button, then
+	# dragged off the menu (possibly with a cascade posted) and
+	# released.  Unpost everything and quit.
+
+	$w postcascade none
+	$w activate none
+	event generate $w <<MenuSelect>>
+	tkMenuUnpost $w
+	return
+    }
+    if {![string compare [$w type active] "cascade"]} {
 	$w postcascade active
 	set menu [$w entrycget active -menu]
 	tkMenuFirstEntry $menu
-    } elseif {[$w type active] == "tearoff"} {
+    } elseif {![string compare [$w type active] "tearoff"]} {
 	tkMenuUnpost $w
 	tkTearOffMenu $w
+    } elseif {![string compare [$w cget -type] "menubar"]} {
+	$w postcascade none
+	$w activate none
+	event generate $w <<MenuSelect>>
+	tkMenuUnpost $w
     } else {
 	tkMenuUnpost $w
 	uplevel #0 [list $w invoke active]
@@ -491,14 +665,53 @@ proc tkMenuInvoke w {
 # menu -		Name of the menu window.
 
 proc tkMenuEscape menu {
-    if {[winfo class [winfo parent $menu]] != "Menu"} {
+    set parent [winfo parent $menu]
+    if {[string compare [winfo class $parent] "Menu"]} {
 	tkMenuUnpost $menu
+    } elseif {![string compare [$parent cget -type] "menubar"]} {
+	tkMenuUnpost $menu
+	tkRestoreOldGrab
     } else {
-	tkMenuLeftRight $menu -1
+	tkMenuNextMenu $menu left
     }
 }
 
-# tkMenuLeftRight --
+# The following routines handle arrow keys. Arrow keys behave
+# differently depending on whether the menu is a menu bar or not.
+
+proc tkMenuUpArrow {menu} {
+    if {![string compare [$menu cget -type] "menubar"]} {
+	tkMenuNextMenu $menu left
+    } else {
+	tkMenuNextEntry $menu -1
+    }
+}
+
+proc tkMenuDownArrow {menu} {
+    if {![string compare [$menu cget -type] "menubar"]} {
+	tkMenuNextMenu $menu right
+    } else {
+	tkMenuNextEntry $menu 1
+    }
+}
+
+proc tkMenuLeftArrow {menu} {
+    if {![string compare [$menu cget -type] "menubar"]} {
+	tkMenuNextEntry $menu -1
+    } else {
+	tkMenuNextMenu $menu left
+    }
+}
+
+proc tkMenuRightArrow {menu} {
+    if {![string compare [$menu cget -type] "menubar"]} {
+	tkMenuNextEntry $menu 1
+    } else {
+	tkMenuNextMenu $menu right
+    }
+}
+
+# tkMenuNextMenu --
 # This procedure is invoked to handle "left" and "right" traversal
 # motions in menus.  It traverses to the next menu in a menu bar,
 # or into or out of a cascaded menu.
@@ -508,47 +721,72 @@ proc tkMenuEscape menu {
 #			event.
 # direction -		Direction in which to move: "left" or "right"
 
-proc tkMenuLeftRight {menu direction} {
+proc tkMenuNextMenu {menu direction} {
     global tkPriv
 
     # First handle traversals into and out of cascaded menus.
 
-    if {$direction == "right"} {
+    if {![string compare $direction "right"]} {
 	set count 1
-	if {[$menu type active] == "cascade"} {
+	set parent [winfo parent $menu]
+	set class [winfo class $parent]
+      if {![string compare [$menu type active] "cascade"]} {
 	    $menu postcascade active
 	    set m2 [$menu entrycget active -menu]
-	    if {$m2 != ""} {
+          if {[string compare $m2 ""]} {
 		tkMenuFirstEntry $m2
 	    }
 	    return
+	} else {
+	    set parent [winfo parent $menu]
+          while {[string compare $parent "."]} {
+              if {![string compare [winfo class $parent] "Menu"]
+                      && ![string compare [$parent cget -type] "menubar"]} {
+		    tk_menuSetFocus $parent
+		    tkMenuNextEntry $parent 1
+		    return
+		}
+		set parent [winfo parent $parent]
+	    }
 	}
     } else {
 	set count -1
 	set m2 [winfo parent $menu]
-	if {[winfo class $m2] == "Menu"} {
-	    $menu activate none
-	    focus $m2
-
-	    # This code unposts any posted submenu in the parent.
-
-	    set tmp [$m2 index active]
-	    $m2 activate none
-	    $m2 activate $tmp
-	    return
+      if {![string compare [winfo class $m2] "Menu"]} {
+          if {[string compare [$m2 cget -type] "menubar"]} {
+		$menu activate none
+		tkGenerateMenuSelect $menu
+		tk_menuSetFocus $m2
+		
+		# This code unposts any posted submenu in the parent.
+		
+		set tmp [$m2 index active]
+		$m2 activate none
+		$m2 activate $tmp
+		return
+	    }
 	}
     }
 
     # Can't traverse into or out of a cascaded menu.  Go to the next
     # or previous menubutton, if that makes sense.
 
+    set m2 [winfo parent $menu]
+    if {![string compare [winfo class $m2] "Menu"]} {
+      if {![string compare [$m2 cget -type] "menubar"]} {
+	    tk_menuSetFocus $m2
+	    tkMenuNextEntry $m2 -1
+	    return
+	}
+    }
+
     set w $tkPriv(postedMb)
-    if {$w == ""} {
+    if {![string compare $w ""]} {
 	return
     }
     set buttons [winfo children [winfo parent $w]]
     set length [llength $buttons]
-    set i [expr [lsearch -exact $buttons $w] + $count]
+    set i [expr {[lsearch -exact $buttons $w] + $count}]
     while 1 {
 	while {$i < 0} {
 	    incr i $length
@@ -557,13 +795,13 @@ proc tkMenuLeftRight {menu direction} {
 	    incr i -$length
 	}
 	set mb [lindex $buttons $i]
-	if {([winfo class $mb] == "Menubutton")
-		&& ([$mb cget -state] != "disabled")
-		&& ([$mb cget -menu] != "")
-		&& ([[$mb cget -menu] index last] != "none")} {
+      if {![string compare [winfo class $mb] "Menubutton"]
+              && [string compare [$mb cget -state] "disabled"]
+              && [string compare [$mb cget -menu] ""]
+              && [string compare [[$mb cget -menu] index last] "none"]} {
 	    break
 	}
-	if {$mb == $w} {
+      if {![string compare $mb $w]} {
 	    return
 	}
 	incr i $count
@@ -583,17 +821,25 @@ proc tkMenuLeftRight {menu direction} {
 
 proc tkMenuNextEntry {menu count} {
     global tkPriv
-    if {[$menu index last] == "none"} {
+
+    if {![string compare [$menu index last] "none"]} {
 	return
     }
-    set length [expr [$menu index last]+1]
+    set length [expr {[$menu index last]+1}]
+    set quitAfter $length
     set active [$menu index active]
-    if {$active == "none"} {
+    if {![string compare $active "none"]} {
 	set i 0
     } else {
-	set i [expr $active + $count]
+	set i [expr {$active + $count}]
     }
     while 1 {
+	if {$quitAfter <= 0} {
+	    # We've tried every entry in the menu.  Either there are
+	    # none, or they're all disabled.  Just give up.
+
+	    return
+	}
 	while {$i < 0} {
 	    incr i $length
 	}
@@ -609,15 +855,25 @@ proc tkMenuNextEntry {menu count} {
 	    return
 	}
 	incr i $count
+	incr quitAfter -1
     }
     $menu activate $i
-    $menu postcascade $i
+    tkGenerateMenuSelect $menu
+    if {![string compare [$menu type $i] "cascade"]} {
+	set cascade [$menu entrycget $i -menu]
+      if {[string compare $cascade ""]} {
+	    $menu postcascade $i
+	    tkMenuFirstEntry $cascade
+	}
+    }
 }
 
 # tkMenuFind --
 # This procedure searches the entire window hierarchy under w for
 # a menubutton that isn't disabled and whose underlined character
-# is "char".  It returns the name of that window, if found, or an
+# is "char" or an entry in a menubar that isn't disabled and whose
+# underlined character is "char".
+# It returns the name of that window, if found, or an
 # empty string if no matching window was found.  If "char" is an
 # empty string then the procedure returns the name of the first
 # menubutton found that isn't disabled.
@@ -631,22 +887,58 @@ proc tkMenuNextEntry {menu count} {
 proc tkMenuFind {w char} {
     global tkPriv
     set char [string tolower $char]
+    set windowlist [winfo child $w]
 
-    foreach child [winfo child $w] {
+    foreach child $windowlist {
+	# Don't descend into other toplevels.
+        if {[winfo toplevel [focus]] != [winfo toplevel $child] } {
+	    continue
+	}
+	switch [winfo class $child] {
+	    Menu {
+              if {![string compare [$child cget -type] "menubar"]} {
+                  if {![string compare $char ""]} {
+			return $child
+		    }
+		    set last [$child index last]
+		    for {set i [$child cget -tearoff]} {$i <= $last} {incr i} {
+                      if {![string compare [$child type $i] "separator"]} {
+			    continue
+			}
+			set char2 [string index [$child entrycget $i -label] \
+				[$child entrycget $i -underline]]
+                      if {![string compare $char [string tolower $char2]] \
+                              || ![string compare $char ""]} {
+                          if {[string compare [$child entrycget $i -state] "disabled"]} {
+				return $child
+			    }
+			}
+		    }
+		}
+	    }
+	}
+    }
+
+    foreach child $windowlist {
+	# Don't descend into other toplevels.
+        if {[winfo toplevel [focus]] != [winfo toplevel $child] } {
+	    continue
+	}
 	switch [winfo class $child] {
 	    Menubutton {
 		set char2 [string index [$child cget -text] \
 			[$child cget -underline]]
-		if {([string compare $char [string tolower $char2]] == 0)
-			|| ($char == "")} {
-		    if {[$child cget -state] != "disabled"} {
+              if {![string compare $char [string tolower $char2]]
+                      || ![string compare $char ""]} {
+                  if {[string compare [$child cget $i -state] "disabled"]} {
 			return $child
 		    }
 		}
 	    }
-	    Frame {
+
+	    default {
 		set match [tkMenuFind $child $char]
-		if {$match != ""} {
+              if {[string compare $match ""]} {
 		    return $match
 		}
 	    }
@@ -668,16 +960,32 @@ proc tkMenuFind {w char} {
 #				happens.
 
 proc tkTraverseToMenu {w char} {
-    if {$char == ""} {
+    global tkPriv
+    if {![string compare $char ""]} {
 	return
     }
-    while {[winfo class $w] == "Menu"} {
+    while {![string compare [winfo class $w] "Menu"]} {
+      if {[string compare [$w cget -type] "menubar"]
+              && ![string compare $tkPriv(postedMb) ""]} {
+	    return
+	}
+      if {![string compare [$w cget -type] "menubar"]} {
+	    break
+	}
 	set w [winfo parent $w]
     }
     set w [tkMenuFind [winfo toplevel $w] $char]
-    if {$w != ""} {
-	tkMbPost $w
-	tkMenuFirstEntry [$w cget -menu]
+    if {[string compare $w ""]} {
+      if {![string compare [winfo class $w] "Menu"]} {
+	    tk_menuSetFocus $w
+	    set tkPriv(window) $w
+	    tkSaveGrabInfo $w
+	    grab -global $w
+	    tkTraverseWithinMenu $w $char
+	} else {
+	    tkMbPost $w
+	    tkMenuFirstEntry [$w cget -menu]
+	}
     }
 }
 
@@ -691,9 +999,17 @@ proc tkTraverseToMenu {w char} {
 
 proc tkFirstMenu w {
     set w [tkMenuFind [winfo toplevel $w] ""]
-    if {$w != ""} {
-	tkMbPost $w
-	tkMenuFirstEntry [$w cget -menu]
+    if {[string compare $w ""]} {
+      if {![string compare [winfo class $w] "Menu"]} {
+	    tk_menuSetFocus $w
+	    set tkPriv(window) $w
+	    tkSaveGrabInfo $w
+	    grab -global $w
+	    tkMenuFirstEntry $w
+	} else {
+	    tkMbPost $w
+	    tkMenuFirstEntry [$w cget -menu]
+	}
     }
 }
 
@@ -709,26 +1025,27 @@ proc tkFirstMenu w {
 #				nothing happens.
 
 proc tkTraverseWithinMenu {w char} {
-    if {$char == ""} {
+    if {![string compare $char ""]} {
 	return
     }
     set char [string tolower $char]
     set last [$w index last]
-    if {$last == "none"} {
+    if {![string compare $last "none"]} {
 	return
     }
     for {set i 0} {$i <= $last} {incr i} {
-	if [catch {set char2 [string index \
+	if {[catch {set char2 [string index \
 		[$w entrycget $i -label] \
-		[$w entrycget $i -underline]]}] {
+		[$w entrycget $i -underline]]}]} {
 	    continue
 	}
-	if {[string compare $char [string tolower $char2]] == 0} {
-	    if {[$w type $i] == "cascade"} {
-		$w postcascade $i
+      if {![string compare $char [string tolower $char2]]} {
+          if {![string compare [$w type $i] "cascade"]} {
 		$w activate $i
+		$w postcascade active
+		event generate $w <<MenuSelect>>
 		set m2 [$w entrycget $i -menu]
-		if {$m2 != ""} {
+              if {[string compare $m2 ""]} {
 		    tkMenuFirstEntry $m2
 		}
 	    } else {
@@ -752,21 +1069,30 @@ proc tkTraverseWithinMenu {w char} {
 # menu -		Name of the menu window (possibly empty).
 
 proc tkMenuFirstEntry menu {
-    if {$menu == ""} {
+    if {![string compare $menu ""]} {
 	return
     }
-    focus $menu
-    if {[$menu index active] != "none"} {
+    tk_menuSetFocus $menu
+    if {[string compare [$menu index active] "none"]} {
 	return
     }
     set last [$menu index last]
-    if {$last == "none"} {
+    if {![string compare $last "none"]} {
 	return
     }
     for {set i 0} {$i <= $last} {incr i} {
 	if {([catch {set state [$menu entrycget $i -state]}] == 0)
-		&& ($state != "disabled") && ([$menu type $i] != "tearoff")} {
+              && [string compare $state "disabled"]
+              && [string compare [$menu type $i] "tearoff"]} {
 	    $menu activate $i
+	    tkGenerateMenuSelect $menu
+          if {![string compare [$menu type $i] "cascade"]} {
+		set cascade [$menu entrycget $i -menu]
+              if {[string compare $cascade ""]} {
+		    $menu postcascade $i
+		    tkMenuFirstEntry $cascade
+		}
+	    }
 	    return
 	}
     }
@@ -790,12 +1116,12 @@ proc tkMenuFindName {menu s} {
 	return $i
     }
     set last [$menu index last]
-    if {$last == "none"} {
+    if {![string compare $last "none"]} {
 	return
     }
     for {set i 0} {$i <= $last} {incr i} {
-	if ![catch {$menu entrycget $i -label} label] {
-	    if {$label == $s} {
+	if {![catch {$menu entrycget $i -label} label]} {
+          if {![string compare $label $s]} {
 		return $i
 	    }
 	}
@@ -816,19 +1142,23 @@ proc tkMenuFindName {menu s} {
 #			upper-left corner goes at (x,y).
 
 proc tkPostOverPoint {menu x y {entry {}}}  {
-    if {$entry != {}} {
+    global tcl_platform
+    
+    if {[string compare $entry {}]} {
 	if {$entry == [$menu index last]} {
-	    incr y [expr -([$menu yposition $entry] \
-		    + [winfo reqheight $menu])/2]
+	    incr y [expr {-([$menu yposition $entry] \
+		    + [winfo reqheight $menu])/2}]
 	} else {
-	    incr y [expr -([$menu yposition $entry] \
-		    + [$menu yposition [expr $entry+1]])/2]
+	    incr y [expr {-([$menu yposition $entry] \
+		    + [$menu yposition [expr {$entry+1}]])/2}]
 	}
-	incr x [expr -[winfo reqwidth $menu]/2]
+	incr x [expr {-[winfo reqwidth $menu]/2}]
     }
     $menu post $x $y
-    if {($entry != {}) && ([$menu entrycget $entry -state] != "disabled")} {
+    if {[string compare $entry {}]
+          && [string compare [$menu entrycget $entry -state] "disabled"]} {
 	$menu activate $entry
+	tkGenerateMenuSelect $menu
     }
 }
 
@@ -843,11 +1173,55 @@ proc tkPostOverPoint {menu x y {entry {}}}  {
 proc tkSaveGrabInfo w {
     global tkPriv
     set tkPriv(oldGrab) [grab current $w]
-    if {$tkPriv(oldGrab) != ""} {
+    if {[string compare $tkPriv(oldGrab) ""]} {
 	set tkPriv(grabStatus) [grab status $tkPriv(oldGrab)]
     }
 }
 
+# tkRestoreOldGrab --
+# Restores the grab to what it was before TkSaveGrabInfo was called.
+#
+
+proc tkRestoreOldGrab {} {
+    global tkPriv
+
+    if {[string compare $tkPriv(oldGrab) ""]} {
+
+    	# Be careful restoring the old grab, since it's window may not
+	# be visible anymore.
+
+	catch {
+          if {![string compare $tkPriv(grabStatus) "global"]} {
+		grab set -global $tkPriv(oldGrab)
+	    } else {
+		grab set $tkPriv(oldGrab)
+	    }
+	}
+	set tkPriv(oldGrab) ""
+    }
+}
+
+proc tk_menuSetFocus {menu} {
+    global tkPriv
+    if {![info exists tkPriv(focus)] || ![string compare $tkPriv(focus) {}]} {
+	set tkPriv(focus) [focus]
+    }
+    focus $menu
+}
+    
+proc tkGenerateMenuSelect {menu} {
+    global tkPriv
+
+    if {![string compare $tkPriv(activeMenu) $menu] \
+          && ![string compare $tkPriv(activeItem) [$menu index active]]} {
+	return
+    }
+
+    set tkPriv(activeMenu) $menu
+    set tkPriv(activeItem) [$menu index active]
+    event generate $menu <<MenuSelect>>
+}
+
 # tk_popup --
 # This procedure pops up a menu and sets things up for traversing
 # the menu and its submenus.
@@ -862,13 +1236,16 @@ proc tkSaveGrabInfo w {
 
 proc tk_popup {menu x y {entry {}}} {
     global tkPriv
-    if {($tkPriv(popup) != "") || ($tkPriv(postedMb) != "")} {
+    global tcl_platform
+    if {[string compare $tkPriv(popup) ""]
+          || [string compare $tkPriv(postedMb) ""]} {
 	tkMenuUnpost {}
     }
     tkPostOverPoint $menu $x $y $entry
-    tkSaveGrabInfo $menu
-    grab -global $menu
-    set tkPriv(popup) $menu
-    set tkPriv(focus) [focus]
-    focus $menu
+    if {![string compare $tcl_platform(platform) "unix"]} {
+	tkSaveGrabInfo $menu
+	grab -global $menu
+	set tkPriv(popup) $menu
+	tk_menuSetFocus $menu
+    }
 }
